from civilpy.structural.steel import W, MC
from civilpy.general import units
from termcolor import colored
import math


class GlobalDefinitions:
    def __init__(
            self,
            f_y=50000 * units('psi'),
            f_u=65000 * units('psi'),
            e_steel=29000000 * units('psi'),
            tie_length=8.5 * units('ft'),
            tie_width=8 * units('inch'),
            tie_depth=7 * units('inch'),
            tie_design_width=8 * units('ft'),  # Design width for distributed loading, length
            max_tie_spacing=24 * units('inch'),  # used for dead load
            future_ballast=12 * units('inch'),
            future_ballast_quantity=1,
            tie_material='Concrete',
            tie_spacing=1.5 * units('ft'),
            railroad_gage=4 * units('ft') + 8.5 * units('inch'),  # //TODO - Verify: Not AREMA
            poisson_ratio=0.30,
            steel_unit_weight=490 * units('lbf/ft^3'),
            ballast_unit_weight=120 * units('lbf/ft^3'),
            tie_unit_weight=150 * units('lbf/ft^3'),
            track_unit_weight=200 * units('lbf/ft'),
            waterproofing_unit_weight=2 * units('lbf/ft^2'),
            steel_connection_contingency=.1,
            asphalt_unit_weight=150 * units('lbf/ft^3'),
            timber_unit_weight=60 * units('lbf/ft^3'),
    ):
        super().__init__()
        self.F_y = f_y
        self.F_u = f_u
        self.E_steel = self.E = e_steel
        self.tie_length = tie_length
        self.tie_width = tie_width
        self.tie_depth = tie_depth
        self.tie_design_width = tie_design_width
        self.max_tie_spacing = max_tie_spacing
        self.future_ballast = future_ballast
        self.future_ballast_quantity = future_ballast_quantity
        self.tie_material = tie_material
        self.tie_spacing = tie_spacing
        self.railroad_gage = railroad_gage
        self.poisson_ratio = poisson_ratio
        self.steel_unit_weight = steel_unit_weight
        self.ballast_unit_weight = ballast_unit_weight
        self.tie_unit_weight = tie_unit_weight
        self.track_unit_weight = track_unit_weight
        self.waterproofing_unit_weight = waterproofing_unit_weight
        self.steel_connection_contingency = steel_connection_contingency
        self.asphalt_unit_weight = asphalt_unit_weight
        self.timber_unit_weight = timber_unit_weight


class LoadDefinitions:
    def __init__(self, diaphragm_weight_ft=61 * units('lbf/ft'),
                 diaphragm_quant=1, bracing_quant=4, fb_a=100 * units('kip'), fb_s=5 * units('ft'),
                 ballasted_deck_reduction=.9, rocking_percent=.2, wind_load=300 * units('lbf/ft'),
                 wind_load_h=8 * units('ft'), axel_load=100 * units('kips'), reduction_fact=.9,
                 wheel_load_percentage=.2,
                 # //TODO - Replace these values with a simpler table lookup
                 e80_50_ft=1901.80 * units('kip*ft'),
                 e80_55_ft=2233.10 * units('kip*ft'),
                 end_diaphragm_weight_per_ft=61 * units('lbf/ft'),
                 end_diaphragm_quantity=1,
                 axel_alternative_live_load=100 * units('kips'),
                 jack_pt_offset=3.75 * units('ft')
                 ):
        self.diaphragm_weight_ft = diaphragm_weight_ft
        self.diaphragm_quant = diaphragm_quant
        self.bracing_quant = bracing_quant
        self.fb_a = fb_a
        self.fb_s = fb_s
        self.ballasted_deck_reduction = ballasted_deck_reduction
        self.rocking_percent = rocking_percent
        self.wind_load = wind_load
        self.wind_load_h = wind_load_h
        self.axel_load = axel_load
        self.reduction_fact = reduction_fact
        self.wheel_load_percentage = wheel_load_percentage
        self.e80_50_ft = e80_50_ft
        self.e80_55_ft = e80_55_ft
        self.end_diaphragm_weight_per_ft = end_diaphragm_weight_per_ft
        self.end_diaphragm_quantity = end_diaphragm_quantity
        self.axel_alternative_live_load = axel_alternative_live_load
        self.jack_pt_offset = jack_pt_offset


class TPG(GlobalDefinitions, LoadDefinitions):
    def __init__(
            self,
            floorbeam_spacing=2.6 * units('ft'),
            floorbeam_quantity=19,
            end_floorbeam_quantity=2,

            # FB Definitions
            # //TODO - Replace the following values with rolled beam values if we usually use rolled beams
            fb_bracket_flange_t=0.75 * units('inch'),
            fb_bracket_flange_web=8.0 * units('inch'),
            fb_bracket_flange_len=3.33 * units('foot'),
            fb_bracket_web_t=.5 * units('inch'),
            fb_bracket_web_width=1.83 * units('ft'),
            fb_bracket_web_height=2.75 * units('ft'),
            fb_bracket_quantity=12,

            # Girder Definitions
            girder_spacing=20 * units('ft'),
            girder_web_height=60 * units.inch,
            girder_web_thickness=0.625 * units.inch,
            girder_flange_width=20 * units.inch,
            girder_flange_thickness=1.625 * units.inch,

            # Diaphragm Definitions
            max_diaphragm_spacing=10.00 * units('ft'),
            diaphragm_quantity=20,

            # Hole Definitions
            diaphragm_dia_hole=1.0 * units('in'),
            diaphragm_num_holes=4,
            end_diaphragm_hole_quantity=6,

            lateral_dia_holes=1.0 * units('in'),
            lateral_num_holes=2,
            web_conn_girder_holes_num=6,
            web_conn_girder_holes_dia=1 * units('in'),

            span_length=52.00 * units('ft'),
            girder_length=54 * units('ft'),
            floor_length=58 * units('ft'),
            floor_quantity=1,
            rail_quantity=1,
            asphaltic_plank_t=1 * units('in'),
            ballast_plate_spacing=3 * units('ft'),
            asphaltic_plank_quant=1,

            # Deck plate definitions
            deck_plate_thickness=0.75 * units('in'),
            min_ballast_below_tie=6 * units('in'),
            deck_plate_width=12.00 * units('in'),

            waterproofing_quant=1,
            ballast_under_ties_t=9 * units('in'),
            ballast_under_ties_quant=1,
            tie_level_ballast_quant=1,
            tie_level_ballast_sloped_reduction=2 * units('ft^2'),
            tie_level_ballast_include_pre_move_weight=False,
            # //TODO - Verify what these diagonal/horizontal formulas/values are dependent on
            dia_stop_pl_t=.5 * units('in'),
            dia_stop_pl_width=((1 * units('ft') + 10 * units('inch')) ** 2 + (
                    1 * units.ft + (1 + 3 / 16) * units('inch')) ** 2) ** .5,
            dia_stop_pl_quantity=40,
            horizontal_upper_floor_pl_t=0.50 * units('in'),
            horizontal_upper_floor_pl_width=((1 + 10 / 12 + 13 / 16 / 12) + 3 / 12) * units('ft'),
            horizontal_upper_floor_pl_quantity=10,
            girder_quantity=2,
            assumed_mean_impact_perc=.35,  # AREMA Table 15-1-8

            # Bracing Values
            bracing_section='MC10x33.6',
            lateral_bracing_length=32.8024 * units('ft'),
            lateral_bracing_quantity=4,

            # Stiffener Values
            trans_stiff_da=96 * units('inch'),
            trans_stiff_actual_da=60 * units.inch,
            stiffener_width_bst=6 * units('in'),
            stiffener_thickness_tst=0.5 * units('in'),
            bearing_stiffener_thickness_tsb=1 * units('in'),
            bearing_stiffener_corner_clip=1 * units('in'),
            bearing_stiffener_fillet_weld_leg=0.3125 * units('in'),

            floorbeam=W('W24x250'),
            end_floorbeam=W('W21X166'),
            diaphragm=W('W16X89'),
            lateral_bracing=MC('MC10x33.6'),

            # Loads Load Definitions
            load_values=LoadDefinitions(),

            # Loads Global Variables (Non-bridge specific)
            global_values=GlobalDefinitions(),

    ):
        GlobalDefinitions.__init__(self, f_y=global_values.F_y, f_u=global_values.F_u,
                                   e_steel=global_values.E_steel,
                                   tie_length=global_values.tie_length,
                                   tie_width=global_values.tie_width,
                                   tie_depth=global_values.tie_depth,
                                   tie_design_width=global_values.tie_design_width,
                                   max_tie_spacing=global_values.max_tie_spacing,
                                   tie_material=global_values.tie_material,
                                   tie_spacing=global_values.tie_spacing,
                                   future_ballast=global_values.future_ballast,
                                   future_ballast_quantity=global_values.future_ballast_quantity,
                                   railroad_gage=global_values.railroad_gage,
                                   poisson_ratio=global_values.poisson_ratio,
                                   steel_unit_weight=global_values.steel_unit_weight,
                                   ballast_unit_weight=global_values.ballast_unit_weight,
                                   tie_unit_weight=global_values.tie_unit_weight,
                                   track_unit_weight=global_values.track_unit_weight,
                                   waterproofing_unit_weight=global_values.waterproofing_unit_weight,
                                   steel_connection_contingency=global_values.steel_connection_contingency,
                                   asphalt_unit_weight=global_values.asphalt_unit_weight,
                                   timber_unit_weight=global_values.timber_unit_weight
                                   )

        LoadDefinitions.__init__(self,
                                 diaphragm_weight_ft=load_values.diaphragm_weight_ft,
                                 diaphragm_quant=load_values.diaphragm_quant,
                                 bracing_quant=load_values.bracing_quant,
                                 fb_a=load_values.fb_a,
                                 fb_s=load_values.fb_s,
                                 ballasted_deck_reduction=load_values.ballasted_deck_reduction,
                                 rocking_percent=load_values.rocking_percent,
                                 wind_load=load_values.wind_load,
                                 wind_load_h=load_values.wind_load_h,
                                 axel_load=load_values.axel_load,
                                 reduction_fact=load_values.reduction_fact,
                                 wheel_load_percentage=load_values.wheel_load_percentage,
                                 # //TODO - Replace these values with a simpler table lookup
                                 e80_50_ft=load_values.e80_50_ft,
                                 e80_55_ft=load_values.e80_55_ft,
                                 end_diaphragm_weight_per_ft=load_values.end_diaphragm_weight_per_ft,
                                 end_diaphragm_quantity=load_values.end_diaphragm_quantity,
                                 axel_alternative_live_load=load_values.axel_alternative_live_load,
                                 jack_pt_offset=load_values.jack_pt_offset
                                 )

        # Global Input
        self.floorbeam_spacing = floorbeam_spacing
        self.floorbeam_quantity = floorbeam_quantity
        self.end_floorbeam_quantity = end_floorbeam_quantity

        # Steel Section inputs
        self.floorbeam = floorbeam
        self.end_floorbeam = end_floorbeam
        self.diaphragm = diaphragm
        self.lateral_bracing = lateral_bracing

        # Floorbeam Values
        # //TODO - update to read from AISC for rolled shapes
        self.fb_bracket_quantity = fb_bracket_quantity
        self.fb_bracket_flange_t = fb_bracket_flange_t
        self.fb_bracket_flange_web = fb_bracket_flange_web
        self.fb_bracket_flange_len = fb_bracket_flange_len
        self.fb_bracket_web_t = fb_bracket_web_t
        self.fb_bracket_web_width = fb_bracket_web_width
        self.fb_bracket_web_height = fb_bracket_web_height

        # Girder Geometrics
        self.girder_spacing = girder_spacing
        self.girder_web_height = girder_web_height
        self.girder_web_thickness = girder_web_thickness
        self.girder_flange_width = girder_flange_width
        self.girder_flange_thickness = girder_flange_thickness
        self.fb_a = self.girder_spacing / 2 - self.railroad_gage / 2

        self.ballast_plate_spacing = ballast_plate_spacing
        self.max_diaphragm_spacing = max_diaphragm_spacing
        self.diaphragm_quantity = diaphragm_quantity
        self.span_length = span_length
        self.girder_length = girder_length
        self.floor_length = floor_length
        self.floor_quantity = floor_quantity
        self.rail_quantity = rail_quantity
        self.waterproofing_quant = waterproofing_quant
        self.asphaltic_plank_t = asphaltic_plank_t
        self.asphaltic_plank_quant = asphaltic_plank_quant
        self.ballast_under_ties_t = ballast_under_ties_t
        self.ballast_under_ties_quant = ballast_under_ties_quant
        self.tie_level_ballast_quant = tie_level_ballast_quant
        self.tie_level_ballast_sloped_reduction = tie_level_ballast_sloped_reduction
        self.tie_level_ballast_include_pre_move_weight = tie_level_ballast_include_pre_move_weight
        self.dia_stop_pl_t = dia_stop_pl_t
        self.dia_stop_pl_width = dia_stop_pl_width
        self.dia_stop_pl_quantity = dia_stop_pl_quantity
        self.horizontal_upper_floor_pl_t = horizontal_upper_floor_pl_t
        self.horizontal_upper_floor_pl_width = horizontal_upper_floor_pl_width
        self.horizontal_upper_floor_pl_quantity = horizontal_upper_floor_pl_quantity
        self.diaphragms = diaphragm.id
        self.girder_quantity = girder_quantity
        self.assumed_mean_impact_perc = assumed_mean_impact_perc

        # Section Hole definitions
        self.diaphragm_dia_hole = diaphragm_dia_hole
        self.diaphragm_num_holes = diaphragm_num_holes

        self.lateral_dia_holes = lateral_dia_holes
        self.lateral_num_holes = lateral_num_holes
        self.web_conn_girder_holes_num = web_conn_girder_holes_num
        self.web_conn_girder_holes_dia = web_conn_girder_holes_dia

        # Bracing Value Definitions
        self.bracing_section = bracing_section
        self.lateral_bracing_length = lateral_bracing_length
        self.lateral_bracing_quantity = lateral_bracing_quantity
        self.end_diaphragm_hole_quantity = end_diaphragm_hole_quantity

        # Stiffener Value Definitions
        self.trans_stiff_da = trans_stiff_da
        self.trans_stiff_actual_da = trans_stiff_actual_da
        self.stiffener_width_bst = stiffener_width_bst
        self.stiffener_thickness_tst = stiffener_thickness_tst
        self.bearing_stiffener_thickness_tsb = bearing_stiffener_thickness_tsb
        self.bearing_stiffener_corner_clip = bearing_stiffener_corner_clip
        self.bearing_stiffener_fillet_weld_leg = bearing_stiffener_fillet_weld_leg

        self.deck_plate_thickness = deck_plate_thickness
        self.min_ballast_below_tie = min_ballast_below_tie
        self.deck_plate_width = deck_plate_width
        self.ballast_plates_clear_space = girder_spacing - 2 * ballast_plate_spacing
        self.stop_plate_detail_width = (girder_spacing - self.ballast_plates_clear_space) / 2

        if self.span_length > 30 * units('ft'):
            self.impact_factor = .35  # AREMA 15 - Table 15-1-8 # //TODO - Copy
        else:
            pass

        self.L_brace = 4 * self.floorbeam_spacing
        # AREMA 15- 1.7.7.a
        self.bearing_stiffener_width_bsb = ((self.girder_flange_width - self.girder_web_thickness) / 2)

        # Basic girder geometrics definitions
        self.girder_top_flange_area = self.girder_flange_width * self.girder_flange_thickness
        self.girder_web_area = self.girder_web_height * self.girder_web_thickness
        self.girder_bot_flange_area = self.girder_flange_width * self.girder_flange_thickness

        self.girder_top_flange_centroid_height = (self.girder_flange_thickness + self.girder_web_height +
                                                  self.girder_flange_thickness / 2)
        self.girder_web_centroid_height = self.girder_flange_thickness + self.girder_web_height / 2
        self.girder_bot_flange_centroid_height = self.girder_flange_thickness / 2

        # Moment of Inertia values
        self.girder_top_flange_I_o = (self.girder_flange_width * self.girder_flange_thickness ** 3) / 12
        self.girder_web_I_o = (self.girder_web_thickness * self.girder_web_height ** 3) / 12
        self.girder_bot_flange_I_o = (self.girder_flange_width * self.girder_flange_thickness ** 3) / 12

        # Table Values depend on the following two definitions
        self.girder_height = self.girder_flange_thickness + self.girder_web_height + self.girder_flange_thickness
        self.girder_centroid = self.girder_height / 2

        # Ay^2 Values in table
        self.girder_top_flange_A_y_sq = self.girder_top_flange_area * (
                    self.girder_top_flange_centroid_height - self.girder_centroid) ** 2
        self.girder_web_A_y_sq = self.girder_web_area * (self.girder_web_centroid_height - self.girder_centroid) ** 2
        self.girder_bot_flange_A_y_sq = self.girder_top_flange_area * (
                    self.girder_bot_flange_centroid_height - self.girder_centroid) ** 2

        # Out-of-Plane I_o
        self.girder_top_flange_oop_I_o = (self.girder_flange_thickness * self.girder_flange_width ** 3) / 12
        self.girder_web_oop_I_o = self.girder_web_height * self.girder_web_thickness ** 3 / 12
        self.girder_bot_flange_oop_I_o = (self.girder_flange_thickness * self.girder_flange_width ** 3) / 12

        # Remaining Girder Section Properties
        self.girder_area = self.girder_top_flange_area + self.girder_web_area + self.girder_bot_flange_area
        self.girder_I_xx = (self.girder_top_flange_I_o + self.girder_web_I_o + self.girder_bot_flange_I_o +
                            self.girder_top_flange_A_y_sq + self.girder_web_A_y_sq + self.girder_bot_flange_A_y_sq)
        self.girder_S_xx = self.girder_I_xx / self.girder_centroid
        self.girder_r_xx = (self.girder_I_xx / self.girder_area) ** .5
        self.girder_I_yy = self.girder_top_flange_oop_I_o + self.girder_web_oop_I_o + self.girder_bot_flange_oop_I_o
        self.girder_S_yy = self.girder_I_yy / (self.girder_flange_width / 2)
        self.girder_r_yy = (self.girder_I_yy / self.girder_area) ** .5
        self.girder_weight_per_ft_no_cont = (self.girder_area * self.steel_unit_weight).to('lbf/ft')

        # No Holes
        self.girder_S_x_net = self.girder_S_xx

        #                                    # Dead Load Calculations
        # Girder Dead Loads
        self.girder_weight_per_ft = (self.girder_weight_per_ft_no_cont *
                                     (1 + self.steel_connection_contingency)).to('kip/ft')
        self.girder_self_load = (
                    self.girder_area * self.girder_length * self.steel_unit_weight * self.girder_quantity * (
                        1 + self.steel_connection_contingency)).to('kip')

        # Intermediate Floor beams Dead Loads
        self.int_floorbeam_weight = (
                    self.girder_spacing *
                    self.floorbeam.weight *
                    self.floorbeam_quantity * (1 + self.steel_connection_contingency)).to('kips')

        # End Floor Beams Dead Loads
        self.end_floorbeam_weight = (
                    self.girder_spacing * self.end_floorbeam.weight *
                    self.end_floorbeam_quantity * (1 + self.steel_connection_contingency)).to('kips')

        self.total_floorbeam_weight = self.end_floorbeam_weight + self.int_floorbeam_weight

        # Diaphragms Dead Loads
        self.total_diaphragm_weight = (
                    self.floorbeam_spacing *
                    self.diaphragm.weight *
                    self.diaphragm_quantity * (1 + self.steel_connection_contingency)).to('kips')

        # Lateral Bracing # //TODO - lateral bracing quantity seems low
        self.total_lateral_weight = (self.lateral_bracing_length *
                                     self.lateral_bracing.weight *
                                     self.lateral_bracing_quantity * (1 + self.steel_connection_contingency)).to('kips')

        # Floor Dead Loads
        self.deck_plate_lin_weight_per_girder = (self.deck_plate_thickness *
                                                 self.ballast_plates_clear_space * self.steel_unit_weight * (
                                                         1 + self.steel_connection_contingency) / 2).to('lbf/ft')
        self.floor_area_load = (self.deck_plate_thickness *
                                self.steel_unit_weight * (1 + self.steel_connection_contingency)).to('lbf/ft^2')

        self.total_floor_weight = (
                    self.deck_plate_thickness *
                    self.ballast_plates_clear_space *
                    self.floor_length *
                    self.steel_unit_weight * self.floor_quantity * (1 + self.steel_connection_contingency)).to('kip')

        # Diagonal Stop Plate # //TODO - ask about what this detail is
        self.dia_stop_pl_lin_w_per_girder = (self.dia_stop_pl_t * self.dia_stop_pl_width * self.steel_unit_weight * (
                    1 + self.steel_connection_contingency)).to('lbf/ft')
        self.dia_stop_pl_area_load = (self.dia_stop_pl_t * self.dia_stop_pl_width * self.steel_unit_weight * (
                    1 + self.steel_connection_contingency) / self.stop_plate_detail_width).to('lbf/ft^2')

        self.dia_stop_pl_weight = (
                    self.dia_stop_pl_t * self.dia_stop_pl_width * self.floorbeam_spacing * self.steel_unit_weight *
                    self.dia_stop_pl_quantity * (1 + self.steel_connection_contingency)).to('kip')

        # Horizontal Upper Floor Plate # //TODO - Ask about what this detail is
        self.horizontal_upper_fl_pl_length = 4 * self.floorbeam_spacing - 8 * units('in')  # //TODO - is this always 8"
        # //TODO - formula given says (1+C), which is hard coded as 10.4, figure out what this is,
        # //TODO - also width has bad math in excel
        self.horizontal_upper_fl_pl_weight_per_girder = (self.horizontal_upper_floor_pl_t *
                                                         self.horizontal_upper_floor_pl_width *
                                                         self.steel_unit_weight *
                                                         (1 + self.steel_connection_contingency) *
                                                         self.horizontal_upper_fl_pl_length /
                                                         (10.4 * units('ft'))).to('lbf/ft')
        self.horizontal_upper_fl_pl_area_load = (self.horizontal_upper_floor_pl_t *
                                                 self.horizontal_upper_floor_pl_width *
                                                 self.steel_unit_weight * (1 + self.steel_connection_contingency) /
                                                 self.stop_plate_detail_width * self.horizontal_upper_fl_pl_length /
                                                 (10.4 * units('ft'))).to('lbf/ft^2')

        self.horizontal_upper_fl_pl_weight = (
                    self.horizontal_upper_floor_pl_t *
                    self.horizontal_upper_floor_pl_width *
                    self.horizontal_upper_fl_pl_length *
                    self.steel_unit_weight * self.horizontal_upper_floor_pl_quantity * (
                        1 + self.steel_connection_contingency)).to('kip')

        self.waterproofing_width = self.ballast_plates_clear_space + 2 * self.dia_stop_pl_width
        self.waterproofing_length = self.floor_length
        self.waterproofing_weight_per_girder = (self.waterproofing_width * self.waterproofing_unit_weight /
                                                self.girder_quantity)
        self.waterproofing_area_load = self.waterproofing_unit_weight
        self.waterproofing_area_load_2 = self.waterproofing_unit_weight * (self.waterproofing_width - 14 * units(
            'ft')) / 2 / self.stop_plate_detail_width
        self.waterproofing_weight = (
                    self.waterproofing_width * self.waterproofing_length *
                    self.waterproofing_unit_weight * self.waterproofing_quant).to('kips')

        # Asphaltic Plank Dead load
        # Asphaltic Plank
        self.asp_plank_width = self.waterproofing_width
        self.asp_plank_length = self.floor_length
        self.asp_lin_weight_per_girder = (self.asp_plank_width * self.asphaltic_plank_t *
                                          self.asphalt_unit_weight / self.girder_quantity).to('lbf/ft')
        self.asp_plank_area_load = (self.asphaltic_plank_t * self.asphalt_unit_weight).to('lbf/ft^2')
        # Between stop Plates        # //TODO - Hardcoded values
        self.asp_plank_area_load_2 = (self.asphaltic_plank_t * self.asphalt_unit_weight * (
                    self.asp_plank_width - 14 * units('ft')) / 2 / self.stop_plate_detail_width).to('lbf/ft^2')

        self.asp_plank_weight = (
                    self.asp_plank_width * self.asp_plank_length * self.asphalt_unit_weight * self.asphaltic_plank_t *
                    self.asphaltic_plank_quant).to('kips')

        # Flooring linear weight per girder
        self.floor_weight_per_girder = self.asp_lin_weight_per_girder + self.waterproofing_weight_per_girder + \
            self.horizontal_upper_fl_pl_weight_per_girder + self.dia_stop_pl_lin_w_per_girder + \
            self.deck_plate_lin_weight_per_girder

        # Flooring Load on FBs between stop plates and deck plate
        self.floor_load_on_fbs = (self.asp_plank_area_load + self.waterproofing_area_load +
                                  self.floor_area_load).to('lbf/ft^2')

        # Flooring Load on FBs over stop plates
        self.floor_load_over_stop_plates = (self.asp_plank_area_load_2 + self.waterproofing_area_load_2 +
                                            self.horizontal_upper_fl_pl_area_load + self.dia_stop_pl_area_load)

        # Ballast
        # Ballast Under Ties
        self.ballast_under_ties_width = (self.ballast_plates_clear_space + 9 / 12 *
                                         self.ballast_under_ties_t * 2 / 2).to('ft')  # //TODO - What is the 9/12?
        self.ballast_under_ties_length = self.floor_length
        self.ballast_volume = (self.ballast_under_ties_width * self.ballast_under_ties_t *
                               self.ballast_under_ties_length).to('ft^3')
        self.ballast_area_load = (
                    self.ballast_under_ties_t * self.ballast_under_ties_width * self.ballast_unit_weight /
                    self.ballast_plates_clear_space).to('lbf/ft^2')
        self.ballast_area_load_2 = (self.ballast_under_ties_t * self.ballast_unit_weight).to('lbf/ft^2')

        self.ballast_weight = (
                    self.ballast_under_ties_t * self.ballast_under_ties_width * self.ballast_under_ties_length *
                    self.ballast_unit_weight * self.ballast_under_ties_quant).to('kips')

        # Tie Dead Load
        self.tie_quantity = self.floor_length / self.tie_spacing
        self.tie_volume = self.tie_depth * self.tie_width * self.tie_length * self.tie_quantity
        self.tie_vol_per_ft = (self.tie_depth * self.tie_width * self.tie_length / self.tie_spacing).to('ft^3/ft')
        self.tie_area_load = (self.tie_depth * self.tie_width * self.tie_length * self.tie_unit_weight * 1 /
                              self.tie_spacing / self.ballast_plates_clear_space).to('lbf / ft^2')
        self.tie_area_load_2 = (self.tie_depth * self.tie_width * self.tie_length * self.tie_unit_weight * 1 /
                                self.tie_spacing / self.tie_length).to('lbf/ft^2')

        self.tie_weight = (self.tie_depth * self.tie_width * self.tie_length *
                           self.tie_unit_weight * self.tie_quantity).to('kips')

        # Ballast at Tie Level
        self.ballast_tie_lvl_t = self.tie_depth
        self.ballast_tie_lvl_width = self.ballast_under_ties_width + 9 / 12 * self.ballast_tie_lvl_t
        self.ballast_tie_level_length = self.ballast_under_ties_length

        self.ballast_tie_lvl_vol = ((self.ballast_tie_lvl_t * self.ballast_tie_lvl_width -
                                     self.tie_level_ballast_sloped_reduction) * self.ballast_tie_level_length -
                                    self.tie_volume).to('ft^3')
        self.ballast_tie_lvl_area_load = ((self.ballast_tie_lvl_t * self.ballast_tie_lvl_width - self.tie_vol_per_ft -
                                           self.tie_level_ballast_sloped_reduction) * self.ballast_unit_weight /
                                          self.ballast_plates_clear_space).to('lbf/ft^2')
        self.ballast_tie_lvl_area_load_2 = (self.ballast_tie_lvl_t * self.ballast_unit_weight * (1 - (
                                            self.tie_width / self.tie_spacing))).to('lbf/ft^2')

        self.ballast_tie_lvl_subtotal = (self.ballast_tie_lvl_t *
                                         self.ballast_tie_lvl_width -
                                         self.tie_level_ballast_sloped_reduction).to('ft^2')

        self.ballast_tie_level_weight = (((self.ballast_tie_lvl_t * self.ballast_tie_lvl_width -
                                           self.tie_level_ballast_sloped_reduction) * self.ballast_tie_level_length -
                                          self.tie_volume) * self.ballast_unit_weight *
                                         self.tie_level_ballast_quant).to('kip')

        # Rail Dead Load
        self.rail_area_load = self.track_unit_weight / self.ballast_plates_clear_space

        # (track_unit_weight / tie_length).to('lbf/ft^2')
        # //TODO - Don't think this is calculating correctly replaced tie width w/ tie length to be closer to same value
        self.rail_area_load_2 = 25 * units('lbf/ft^2')

        self.rail_weight = (self.track_unit_weight * self.floor_length * self.rail_quantity).to('kip')

        # Future ballast Allowance/Dead Load

        self.future_ballast_width = self.ballast_plates_clear_space + 9 / 12 * (
                self.ballast_under_ties_t + self.future_ballast)

        self.future_ballast_vol = (self.future_ballast *
                                   self.future_ballast_width *
                                   self.ballast_tie_level_length).to('ft^3')
        self.future_ballast_area_load = (
                self.future_ballast *
                self.future_ballast_width *
                self.ballast_unit_weight /
                self.ballast_plates_clear_space).to('lbf/ft^2')

        self.future_ballast_area_load_2 = (self.future_ballast * self.ballast_unit_weight).to('lbf/ft^2')

        self.future_ballast_weight = (self.future_ballast_vol *
                                      self.ballast_unit_weight *
                                      self.future_ballast_quantity).to('kip')

        # Summary Calcs
        self.track_weight_per_girder = (
            self.ballast_weight +
            self.tie_weight +
            self.ballast_tie_level_weight +
            self.rail_weight +
            self.future_ballast_weight) / self.ballast_tie_level_length
        self.track_area_load_on_fbs = sum(
            [self.ballast_area_load,
             self.tie_area_load,
             self.ballast_tie_lvl_area_load,
             self.rail_area_load,
             self.future_ballast_area_load
             ]
        )

        self.track_assembly_load_on_deck_plate = sum(
            [self.ballast_area_load_2, self.tie_area_load_2, self.ballast_tie_lvl_area_load_2, self.rail_area_load_2,
             self.future_ballast_area_load_2]).to('lbf/ft^2')

        self.fb_bracket_vol = (
                self.fb_bracket_flange_t *
                self.fb_bracket_flange_web *
                self.fb_bracket_flange_len +
                self.fb_bracket_web_t *
                self.fb_bracket_web_width *
                self.fb_bracket_web_height).to('ft^3')

        self.floorbeam_bracket_weight = (
                    (self.fb_bracket_vol *
                     self.steel_unit_weight *
                     self.fb_bracket_quantity) * (
                     1 + self.steel_connection_contingency)).to('kip')

        self.list_of_dead_load_factors = [
            self.floorbeam_bracket_weight,
            self.future_ballast_weight,
            self.rail_weight,
            self.ballast_tie_level_weight,
            self.tie_weight,
            self.ballast_weight,
            self.asp_plank_weight,
            self.waterproofing_weight,
            self.horizontal_upper_fl_pl_weight,
            self.dia_stop_pl_weight,
            self.total_floor_weight,
            self.total_lateral_weight,
            self.total_diaphragm_weight,
            self.total_floorbeam_weight,
            self.girder_self_load
        ]

        self.lifting_dead_load_factors = [
            self.floorbeam_bracket_weight,
            self.rail_weight,
            self.tie_weight,
            self.ballast_weight,  # //TODO - Shouldn't this not be included here?
            self.asp_plank_weight,
            self.waterproofing_weight,
            self.horizontal_upper_fl_pl_weight,
            self.dia_stop_pl_weight,
            self.total_floor_weight,
            self.total_lateral_weight,
            self.total_diaphragm_weight,
            self.total_floorbeam_weight,
            self.girder_self_load
        ]

        # Calculate the Bridges Deadloads
        self.total_dead_load = sum(self.list_of_dead_load_factors)
        self.total_dead_load_lifting = sum(self.lifting_dead_load_factors)

        self.distributed_load = self.total_dead_load / self.span_length / self.girder_quantity

        # Girder Calcs and Checks
        self.floorbeam_load_on_girder = self.total_floorbeam_weight / self.girder_quantity / self.span_length
        self.diaphragm_load_on_girder = self.total_diaphragm_weight / self.girder_quantity / self.span_length

        self.lat_bracing_load_on_girder = self.total_lateral_weight / self.girder_quantity / self.span_length

        self.track_assembly = self.track_weight_per_girder
        self.total_dl_over_total_length = sum(
            [self.girder_weight_per_ft, self.floorbeam_load_on_girder, self.diaphragm_load_on_girder,
             self.floor_weight_per_girder, self.lat_bracing_load_on_girder, self.track_assembly]
        )

        self.w = self.total_dl_over_total_length
        self.L = self.span_length

        self.M_dl = (self.w * self.L ** 2 / 8).to('kips*ft')

        # //TODO - Values are hardcoded
        self.span_live_load = (self.e80_55_ft - self.e80_50_ft) / (55 * units.ft - 50 * units.ft) * (
                    self.span_length - 50 * units.ft) + self.e80_50_ft
        self.M_ll = self.span_live_load

        # Impact Load      AREMA 15.1.3.5.d
        # //TODO - Hardcoded Values, make sure they match AREMA Formula
        self.impact_percent = (40 - 3 * self.span_length ** 2 / (
                    1600 * units('ft^2'))) / 100

        # //TODO - Dimensionality off, /100 should adjustment probably be in the percentage calc
        self.M_i = (self.M_ll * self.ballasted_deck_reduction * self.impact_percent)

        # Rocking Effect    AREMA 15.1.3.5.d
        self.eqiv_uniform_live_load = 8 * self.M_ll / self.span_length ** 2

        self.P = self.eqiv_uniform_live_load * self.rocking_percent

        self.a = (self.girder_spacing - self.railroad_gage) / 2

        self.load_on_girder_rocking = self.P * (1 - 2 * self.a / self.girder_spacing)

        self.M_re = self.load_on_girder_rocking * self.L ** 2 / 8

        # Centrifugal Force    AREMA 15-1.3.6
        # //TODO - Hardcoded Values, should be inputs
        self.design_speed = 35 * units('miles/hour')

        # //TODO - Hardcoded Values, should be inputs
        self.curve_radius = 2000 * units('ft')

        # //TODO - Find a better way to handle units
        self.deg_curve = 2 * math.atan(100 / (2 * self.curve_radius.magnitude)) * units('rad')
        self.centrifugal_percentage = (0.00117 * self.design_speed ** (2 * self.deg_curve))

        # Wind Load    # AREMA 15-1.3.7
        self.wind_load_8_ft_offset = 300 * units('lbf/ft')
        self.P = 8 * units('ft') * self.wind_load_8_ft_offset / self.railroad_gage
        self.a = (self.girder_spacing - self.railroad_gage) / 2

        self.girder_reaction_from_wind = (self.P * (1 - 2 * self.a / self.girder_spacing)).to('kip/ft')

        self.V_wind = self.girder_reaction_from_wind * self.span_length / 2
        self.M_wind = self.girder_reaction_from_wind * self.span_length ** 2 / 8

        # Case 1 = M_dl + M_ll + M_i + M_re
        self.case_1 = sum([self.M_dl, self.M_ll, self.M_i, self.M_re])

        # Case 2 = (M_dl + M_ll + M_i + M_wind) / 1.25
        self.case_2 = sum([self.M_dl, self.M_ll, self.M_i, self.M_wind]) / 1.25

        self.M_tot = max(self.case_1, self.case_2)

        # Req. Section Modulus
        self.F_b = (0.55 * self.F_y).to('kips/in^2')
        self.S_req_moment = (self.M_tot / self.F_b).to('in^3')

        # Fatigue Section Required
        # AREMA 15 - Table 15-1-7, Table 15-1-8
        self.M_if = self.M_i * self.impact_percent
        self.M_f = self.M_ll + self.M_if

        self.F_b_fat = 16 * units('ksi')  # AREMA 15 - Table 15-1-10

        self.S_req_fat = (self.M_f / self.F_b_fat).to('in^3')

        # Required Section Modulus
        self.S_req = max(self.S_req_moment, self.S_req_fat)

        # Deflection requirements for member sizing
        # AREMA 15-1.2.5b
        self.delta_max = (self.span_length / 640).to('inch')

        self.M_ll_and_M_i = self.M_ll + self.M_i

        self.w_deflection = (self.M_ll_and_M_i * 8 / self.span_length ** 2).to('kip/in')

        self.I_x_req = (5 * self.w_deflection * self.span_length ** 4 / (384 * self.E_steel *
                                                                         self.delta_max)).to('in^4')

        #                # AREMA Requirements Checks
        # Flange Compression Check
        if (self.L_brace / self.girder_r_yy) <= 5.55 * (self.E_steel / self.F_y) ** .5:
            print(colored('OK', 'green'))
        else:
            print(colored('No Good - Compression Flange Check Failed', 'red'))

        # Web Thickness     AREMA 15-1.7.3
        # Web Thickness Check 1
        if self.girder_web_thickness >= 0.18 * (self.F_y / self.E) ** .5 * self.girder_web_height:
            print(colored('OK', 'green'))
        else:
            print(colored('No Good - Web Compression Check Failed', 'red'))

        # Web Thickness Check 2
        if self.girder_web_thickness >= self.girder_flange_thickness / 6:
            print(colored('OK', 'green'))
        else:
            print(colored('No Good - Web Compression Check Failed', 'red'))

        # Outstanding Elements in Compression (AREMA 15-1.6.2)
        # Flange_compression
        if self.girder_flange_width / 2 <= 0.43 * self.girder_flange_thickness * (self.E / self.F_y) ** .5:
            print(colored('OK', 'green'))
        else:
            print(colored('No Good - Compression Flange Check Failed', 'red'))

        # Allowable Stress
        # AREMA Table 15-1-12
        self.F_b_ten = 0.55 * self.F_y
        self.comp_value_1 = 0.55 * self.F_y - ((0.55 * self.F_y ** 2) / (6.3 * math.pi ** 2 * self.E_steel)) * \
            (self.L_brace / self.girder_r_yy) ** 2

        self.girder_depth = self.girder_web_height + 2 * self.girder_flange_thickness
        self.girder_flange_area = self.girder_flange_width * self.girder_flange_thickness

        self.comp_value_2 = ((0.131 * math.pi * self.E_steel) / (
                    (self.L_brace * self.girder_depth * (1 + self.poisson_ratio) ** .5) /
                    self.girder_flange_area)).to('psi')

        self.comp_value_3 = 0.55 * self.F_y

        self.F_b_comp = min(self.comp_value_3, max(self.comp_value_2, self.comp_value_1))

        # Stress in Extreme fibers of Girder
        self.F_b_comp_act = (self.M_tot / self.girder_S_xx).to('lbf/in^2')

        # Bending Check
        if self.F_b_comp_act <= self.F_b_comp:
            print(colored(f'OK, Stress Ratio: {self.F_b_comp_act / self.F_b_comp}', 'green'))
        else:
            print(colored('No Good - Girder Bending Check Failed', 'red'))

        self.F_b_ten_act = (self.M_tot / self.girder_S_xx).to('psi')

        # Bending Check
        if self.F_b_ten_act <= self.F_b_ten:
            print(colored(f'OK, Stress Ratio: {self.F_b_ten_act / self.F_b_ten}', 'green'))
        else:
            print(colored('No Good - Girder Bending Check Failed', 'red'))

        # Fatigue (AREMA 15-1.3.13)
        self.S_r_fat_actual = (self.M_f / self.girder_S_xx).to('psi')

        self.S_r_fat = self.F_b_fat.to('psi')

        # Bending Fatigue
        if self.S_r_fat_actual <= self.S_r_fat:
            print(colored(f'OK, Stress Ratio: {self.S_r_fat_actual / self.S_r_fat}', 'green'))
        else:
            print(colored('No Good - Girder Bending Fatigue Check Failed', 'red'))

        # Deflection AREMA 15-1.2.5
        self.w = ((self.M_ll + self.M_i) / (span_length ** 2) * 8)

        self.delta_total = ((5 * self.w * (span_length ** 4)) / (384 * self.E_steel * self.girder_I_xx)).to('in')

        # Deflection # //TODO - Not sure the ratio matters as much for this one
        if self.delta_total <= self.delta_max:
            print(colored(f'OK, Ratio: {self.delta_total / self.delta_max}', 'green'))
        else:
            print(colored('No Good - Deflection Check Failed', 'red'))

        # Dead Load Camber
        self.w_dl = self.total_dl_over_total_length.to('kip/in')
        self.delta_dl = (self.delta_total * self.w_dl / self.w).to('in')

        # Web Shear
        self.V_dl = (self.w_dl * self.span_length / 2).to('kip')

        # Shear from AREMA 15 Table 1-16
        self.shear_50_ft = 174.4 * units('kips')
        self.shear_55_ft = 185.31 * units('kips')
        self.V_ll = (self.shear_55_ft - self.shear_50_ft) / (55 * units('ft') - 50 * units('ft')) * (
                    self.span_length - 50 * units('ft')) + self.shear_50_ft

        self.V_imp = 0.9 * self.V_ll * self.impact_percent  # AREMA 15-1.3.5.b

        self.R1_rocking_force = self.load_on_girder_rocking
        self.V_re = self.R1_rocking_force * self.L / 2

        self.V_tot = self.V_dl + self.V_ll + self.V_re + self.V_imp

        self.F_r = self.V_tot / (self.girder_web_height * self.girder_web_thickness)

        self.F_v = (0.35 * self.F_y).to('kips/in^2')

        # Web Shear AREMA Table 15-1-12 check #
        # // TODO - This Check isn't comparing like units in the excel sheet?, think it should be as follows,
        if self.F_r <= self.F_v:
            print(colored(f'OK, Stress Ratio: {self.F_r / self.F_v}', 'green'))
        else:
            print(colored('No Good - Web Shear Check Failed', 'red'))

        # Allowable Stress on welds
        self.Q = self.girder_flange_width * self.girder_flange_thickness * (self.girder_web_height / 2 +
                                                                            self.girder_flange_thickness / 2)
        self.F_r = self.V_tot * self.Q / (self.girder_I_xx * self.girder_web_thickness)

        # Weld Strength
        if self.F_r <= self.F_v:
            print(colored(f'OK, Stress Ratio: {self.F_r / self.F_v}', 'green'))
        else:
            print(colored('No Good - Weld Strength Check Failed', 'red'))

        # Web plate stiffeners
        # AREMA 15-1.7.8.a
        self.trans_stiff_check = (2.12 * (self.E_steel / self.F_y) ** .5 * self.girder_web_thickness)

        # Transverse stiffener check
        if self.trans_stiff_check <= self.girder_web_height:
            print(colored('Transverse Stiffeners Required', 'red'))
        else:
            print(colored('Transverse Stiffeners NOT Required', 'green'))

        # //TODO - Look into this value, it's not used
        self.clear_dist_to_prev_web_shear_buck = 1.95 * (self.E_steel * self.girder_S_xx) ** .5

        self.Q = self.girder_flange_width * self.girder_flange_thickness * (
                    self.girder_web_height + self.girder_flange_thickness) / 2 + self.girder_web_height * \
            self.girder_web_thickness / 2 * self.girder_web_height / 4

        self.S = self.f_v = self.V_tot * self.Q / (self.girder_I_xx * self.girder_web_thickness)

        self.d = 1.95 * self.girder_web_thickness * (self.E_steel / self.S) ** .5

        self.max_clear_dist = min([self.d, self.trans_stiff_da, self.girder_web_height])

        # Weld Strength
        if self.trans_stiff_actual_da <= self.max_clear_dist or self.girder_web_height <= self.trans_stiff_check:
            print(colored('OK', 'green'))
        else:
            print(colored('No Good - Stiffener Spacing Check Failed', 'red'))

        self.D_d = max(1, min(5, self.girder_web_height / self.d))

        # Required Moment of inertia (Stiffener) AREMA 15-1.7.8.a
        self.stiff_I_xx_req = 2.5 * self.trans_stiff_actual_da * self.girder_web_thickness ** 3 * (self.D_d ** 2 - 0.7)

        self.stiff_I_xx_act = self.stiffener_thickness_tst * self.stiffener_width_bst ** 3 / 12 + \
            self.stiffener_thickness_tst * self.stiffener_width_bst * (
                    self.girder_web_thickness / 2 + self.stiffener_width_bst / 2) ** 2

        # Stiffener I_xx Check
        if self.stiff_I_xx_act >= self.stiff_I_xx_req or self.girder_web_height <= self.trans_stiff_check:
            print(colored('OK', 'green'))
        else:
            print(colored('No Good - Stiffener Spacing Check Failed', 'red'))

        # Max Stiffener Width AREMA 15-1.7.8.b
        self.max_stiff_width = 16 * self.stiffener_thickness_tst
        self.min_stiff_width = 2 * units('in') + self.girder_height / 30

        self.stiff_t_check_1 = self.max_stiff_width >= self.stiffener_width_bst >= self.min_stiff_width
        self.stiff_t_check_2 = self.girder_web_height <= self.trans_stiff_check

        # Stiffener Thickness Check
        if self.stiff_t_check_1 or self.stiff_t_check_2:
            print(colored('OK', 'green'))
        else:
            print(colored('No Good - Stiffener Thickness Check Failed', 'red'))

        # Transverse Stiffener Weld Fatigue Stress
        self.d = self.girder_web_height / 2
        self.S_stiff = self.girder_I_xx / self.d
        self.sr_weld = (self.M_f / self.S_stiff).to('psi')

        self.F_b_fat_trans = 12000 * units('psi')  # Stress category C' N>2,000,000 cycles AREMA 15-1-9

        # Stiffener Weld Fatigue
        if self.sr_weld <= self.F_b_fat_trans:
            print(colored(f'OK, Stress Ratio: {self.sr_weld / self.F_b_fat_trans}', 'green'))
        else:
            print(colored('No Good - Stiffener Weld Fatigue Check Failed', 'red'))

        # Longitudinal Plate Stiffeners
        # AREMA 15-1.7.8.f

        # Longitudinal Stiffeners
        if self.girder_web_height <= 4.18 * (self.E_steel / self.F_y) ** .5 * self.girder_web_thickness:
            self.long_stiff_check = 'Not Required'
            print(colored('Longitudinal Stiffeners not required', 'green'))
        else:
            print(colored('Longitudinal Stiffeners Required', 'red'))

        # Longitudinal Stiffeners   # //TODO - Bad Check
        if self.long_stiff_check == 'Not Required':
            print(colored('OK', 'green'))
        else:
            print(colored('No Good - Longitudinal Stiffeners Check', 'red'))

        # Bearing Stiffener
        self.bearing_stiff_limiting_ratio = (0.43 * self.bearing_stiffener_thickness_tsb *
                                             ((self.E_steel / self.F_y) ** 0.5)).to('in')

        # Longitudinal Stiffeners   # //TODO - Bad Check
        if self.bearing_stiffener_width_bsb <= self.bearing_stiff_limiting_ratio:
            print(colored('OK', 'green'))
        else:
            print(colored('No Good - Bearing Stiffeners w/t ratio Check', 'red'))

        # Outstanding element in compression check
        self.effective_web_length = 25 * self.girder_web_thickness  # AREMA 15-1.7.7.c

        self.bearing_stiff_area = self.girder_web_thickness * self.effective_web_length + 2 * \
                                  self.bearing_stiffener_width_bsb * self.bearing_stiffener_thickness_tsb
        self.I_xx_web = 2 * (
                    self.bearing_stiffener_width_bsb ** 3 * self.bearing_stiffener_thickness_tsb / 12 +
                    self.bearing_stiffener_width_bsb * self.bearing_stiffener_thickness_tsb * (
                        self.girder_web_thickness / 2 + self.bearing_stiffener_width_bsb / 2) ** 2) + \
            self.girder_web_thickness ** 3 * self.effective_web_length / 12
        self.I_xx_stiff = 2 * self.bearing_stiffener_thickness_tsb ** 3 * self.bearing_stiffener_width_bsb / 12 + \
            self.effective_web_length ** 3 * self.girder_web_thickness / 12

        self.controlling_moment_of_inertia = min(self.I_xx_web, self.I_xx_stiff)

        # AREMA 15-1.7.7-c
        self.effective_length = 0.75 * self.girder_web_height
        self.bear_stiff_r_y = (self.controlling_moment_of_inertia / self.bearing_stiff_area) ** 0.5

        self.bear_stiff_slenderness_ratio = self.effective_length / self.bear_stiff_r_y

        # AREMA Table 15-1-12
        self.case_1_all_stress = 0.55 * self.F_y
        self.case_2_all_stress = (0.6 * self.F_y.magnitude - (17500 * self.F_y / self.E_steel) ** (
                    3 / 2) * self.bear_stiff_slenderness_ratio) * units('psi')
        self.case_3_all_stress = 0.514 * math.pi ** 2 * self.E_steel / self.bear_stiff_slenderness_ratio ** 2

        # Controlling Case
        if self.bear_stiff_slenderness_ratio <= 0.629 / (self.F_y / self.E) ** .5:
            print(colored('Case 1 Controls', 'green'))
            self.bearing_stiff_F_a = self.case_1_all_stress
            print(f"bearing stiffener stress: {self.bearing_stiff_F_a}")
        elif self.bear_stiff_slenderness_ratio <= 5.034 / (self.F_y / self.E) ** .5:
            print(colored('Case 2 Controls', 'green'))
            self.bearing_stiff_F_a = self.case_2_all_stress
            print(f"Bearing stiffener stress: {self.bearing_stiff_F_a}")
        else:
            print(colored('Case 3 Controls', 'green'))
            self.bearing_stiff_F_a = self.case_3_all_stress
            print(f"Bearing stiffener stress: {self.bearing_stiff_F_a}")
